@@ -22,7 +22,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -39,6 +39,7 @@
 #include <kmessagebox.h>
 #include <kmimetype.h>
 #include <klocale.h>
+#include <kpassdlg.h>
 #include <kprocess.h>
 #include <kstandarddirs.h>
 
@@ -46,6 +47,7 @@
 #include "arch.h"
 #include "arkwidget.h"
 #include "arkutils.h"
+#include "filelistview.h"
 
 // the archive types
 #include "tar.h"
@@ -56,6 +58,7 @@
 #include "rar.h"
 #include "ar.h"
 #include "sevenzip.h"
+#include "ace.h"
 
 Arch::ArchColumns::ArchColumns( int col, QRegExp reg, int length, bool opt )
   : colRef( col ), pattern( reg ), maxLength( length ), optional( opt )
@@ -68,13 +71,15 @@ Arch::Arch( ArkWidget *gui, const QString &filename )
     m_header_removed( false ), m_finished( false ),
     m_numCols( 0 ), m_dateCol( -1 ), m_fixYear( -1 ), m_fixMonth( -1 ),
     m_fixDay( -1 ), m_fixTime( -1 ), m_repairYear( -1 ), m_repairMonth( -1 ),
-    m_repairTime( -1 )
+    m_repairTime( -1 ), m_currentProcess( 0 )
 {
   m_archCols.setAutoDelete( true ); // To check: it still leaky here???
 }
 
 Arch::~Arch()
 {
+    if ( m_currentProcess )
+        m_currentProcess->kill();
 }
 
 void Arch::verifyUtilityIsAvailable( const QString &utility1,
@@ -98,7 +103,7 @@ void Arch::verifyUtilityIsAvailable( const QString &utility1,
 void Arch::slotOpenExited( KProcess* _kp )
 {
   int exitStatus = 100; // arbitrary bad exit status
-  
+
   if ( _kp->normalExit() )
     exitStatus = _kp->exitStatus();
 
@@ -115,7 +120,7 @@ void Arch::slotOpenExited( KProcess* _kp )
     emit sigOpen( this, false, QString::null, 0 );
 
   delete _kp;
-  _kp = 0;
+  _kp = m_currentProcess = 0;
 }
 
 void Arch::slotDeleteExited( KProcess *_kp )
@@ -125,9 +130,9 @@ void Arch::slotDeleteExited( KProcess *_kp )
   if ( !success )
   {
     QApplication::restoreOverrideCursor();
-    
+
     QString msg = i18n( "The deletion operation failed." );
-    
+
     if ( !getLastShellOutput().isNull() )
     {
       msg += i18n( "\nUse \"Details\" to view the last shell output." );
@@ -141,7 +146,7 @@ void Arch::slotDeleteExited( KProcess *_kp )
 
   emit sigDelete( success );
   delete _kp;
-  _kp = 0;
+  _kp = m_currentProcess = 0;
 }
 
 void Arch::slotExtractExited( KProcess *_kp )
@@ -150,24 +155,54 @@ void Arch::slotExtractExited( KProcess *_kp )
 
   if( !success )
   {
-    QApplication::restoreOverrideCursor();
-    
-    QString msg = i18n( "The extraction operation failed." );
-    
-    if ( !getLastShellOutput().isNull() )
+    if ( passwordRequired() )
     {
-      msg += i18n( "\nUse \"Details\" to view the last shell output." );
-      KMessageBox::detailedError( m_gui, msg, getLastShellOutput() );
+        QString msg;
+        if ( !m_password.isEmpty() )
+            msg = i18n("The password was incorrect. ");
+        if (KPasswordDialog::getPassword( m_password, msg+i18n("You must enter a password to extract the file:") ) == KPasswordDialog::Accepted )
+        {
+            delete _kp;
+            _kp = m_currentProcess = 0;
+            clearShellOutput();
+            unarchFileInternal(); // try to extract the file again with a password
+            return;
+        }
+        emit sigExtract( true );
+        delete _kp;
+        _kp = m_currentProcess = 0;
+        return;
     }
-    else
+    else if ( m_password.isEmpty() || _kp->exitStatus() > 1 )
     {
-      KMessageBox::error( m_gui, msg );
+        QApplication::restoreOverrideCursor();
+
+        QString msg = i18n( "The extraction operation failed." );
+
+        if ( !getLastShellOutput().isNull() )
+        {
+            msg += i18n( "\nUse \"Details\" to view the last shell output." );
+            KMessageBox::detailedError( m_gui, msg, getLastShellOutput() );
+        }
+        else
+        {
+            KMessageBox::error( m_gui, msg );
+        }
     }
   }
-
+  m_password = "";
   emit sigExtract( success );
   delete _kp;
-  _kp = 0;
+  _kp = m_currentProcess = 0;
+}
+
+void Arch::unarchFile( QStringList *fileList, const QString & destDir,
+                         bool viewFriendly )
+{
+    m_fileList = fileList;
+    m_destDir = destDir;
+    m_viewFriendly = viewFriendly;
+    unarchFileInternal();
 }
 
 void Arch::slotAddExited( KProcess *_kp )
@@ -177,9 +212,9 @@ void Arch::slotAddExited( KProcess *_kp )
   if( !success )
   {
     QApplication::restoreOverrideCursor();
-    
+
     QString msg = i18n( "The addition operation failed." );
-    
+
     if ( !getLastShellOutput().isNull() )
     {
       msg += i18n( "\nUse \"Details\" to view the last shell output." );
@@ -193,7 +228,7 @@ void Arch::slotAddExited( KProcess *_kp )
 
   emit sigAdd( success );
   delete _kp;
-  _kp = 0;
+  _kp = m_currentProcess = 0;
 }
 
 void Arch::slotReceivedOutput( KProcess*, char* data, int length )
@@ -255,7 +290,7 @@ void Arch::slotReceivedTOC( KProcess*, char* data, int length )
 
     m_buffer = "";
   }
-  
+
   if ( !m_finished )
     m_buffer.append( data + startChar);	// Append what's left of the buffer
 
@@ -312,12 +347,12 @@ bool Arch::processLine( const QCString &line )
   }
 
   QStringList list;
-  
+
   for ( int i = 0; i < m_numCols; ++i )
   {
     list.append( columns[ i ] );
   }
-  
+
   m_gui->fileList()->addItem( list ); // send the entry to the GUI
 
   return true;
@@ -335,29 +370,31 @@ Arch *Arch::archFactory( ArchType aType,
 
     case ZIP_FORMAT:
       return new ZipArch( parent, filename );
-    
+
     case LHA_FORMAT:
       return new LhaArch( parent, filename );
-    
+
     case COMPRESSED_FORMAT:
       return new CompressedFile( parent, filename, openAsMimeType );
-    
+
     case ZOO_FORMAT:
       return new ZooArch( parent, filename );
-    
+
     case RAR_FORMAT:
       return new RarArch( parent, filename );
-    
+
     case AA_FORMAT:
       return new ArArch( parent, filename );
-    
+
     case SEVENZIP_FORMAT:
       return new SevenZipArch( parent, filename );
-    
+
+    case ACE_FORMAT:
+      return new AceArch( parent, filename );
+
     case UNKNOWN_FORMAT:
     default:
       return 0;
   }
 }
-
 #include "arch.moc"
