@@ -2,6 +2,7 @@
   Copyright (C)
 
   2001: Macadamian Technologies Inc (author: Jian Huang, jian@macadamian.com)
+  2005: Henrique Pinto <henrique.pinto@kdemail.net>
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -15,14 +16,12 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
-#ifndef __ark_part_h__
-#define __ark_part_h__
-
-#include "arkwidget.h"
+#ifndef ARK_PART_H
+#define ARK_PART_H
 
 #include <kparts/part.h>
 #include <kparts/browserextension.h>
@@ -33,8 +32,13 @@
 
 #include <qlabel.h>
 
-
 class KAboutData;
+class ArkWidget;
+
+namespace KIO
+{
+	class Job;
+}
 
 
 class ArkBrowserExtension: public KParts::BrowserExtension
@@ -53,10 +57,13 @@ public:
     ArkStatusBarExtension( KParts::ReadWritePart * parent );
     ~ArkStatusBarExtension();
 
+    void setProgress( unsigned long progress );
+    QPushButton* cancelButton() const { return m_cancelButton; }
+
 public slots:
     void slotSetStatusBarSelectedFiles( const QString & text );
     void slotSetStatusBarText( const QString & text );
-    void slotSetBusy( const QString & text );
+    void slotSetBusy( const QString & text, bool showCancelButton = false, bool detailedProgress = false );
     void slotSetReady();
     void slotProgress();
 
@@ -68,6 +75,7 @@ private:
     QLabel *m_pStatusLabelSelect; // How many files are selected
     QLabel *m_pStatusLabelTotal;  // How many files in archive
     QLabel *m_pBusyText;
+    QPushButton *m_cancelButton; // Cancel an operation
     KProgress *m_pProgressBar;
     QTimer *m_pTimer;
 };
@@ -91,6 +99,11 @@ public slots:
     bool saveFile();
     bool openURL( const KURL & url );
     bool closeURL();
+    void transferStarted( KIO::Job * );
+    void transferCompleted();
+    void transferCanceled( const QString& errMsg );
+    void progressInformation( KIO::Job *, unsigned long );
+    void cancelTransfer();
 
 signals:
     void fixActionState( const bool & bHaveFiles );
@@ -118,7 +131,6 @@ private:
     KAction *viewAction;
     KAction *helpAction;
     KAction *openWithAction;
-    KAction *selectAction;
     KAction *deselectAllAction;
     KAction *invertSelectionAction;
     KAction *editAction;
@@ -126,9 +138,9 @@ private:
     // the following have different enable rules from the above KActions
     KAction *popupViewAction;
     KAction *popupOpenWithAction;
-    KAction *shellOutputAction;
     KToggleAction *showSearchBar;
 
+    KIO::Job *m_job;
 };
 
-#endif
+#endif // ARK_PART_H
