@@ -1,6 +1,6 @@
 /*
 
- $Id: arkutils.cpp 345087 2004-09-08 20:19:07Z henrique $
+ $Id: arkutils.cpp 490596 2005-12-22 13:05:45Z scripty $
 
  ark -- archiver for the KDE project
 
@@ -25,7 +25,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
