@@ -15,25 +15,32 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  */
- 
+
 #include "arkviewer.h"
 
 #include <klocale.h>
 #include <kparts/componentfactory.h>
 #include <kmimetype.h>
 #include <kdebug.h>
+#include <kurl.h>
+#include <kglobal.h>
+#include <kiconloader.h>
 
 #include <qvbox.h>
-#include <qstring.h>
-#include <kurl.h>
+#include <qlayout.h>
+#include <qlabel.h>
+#include <qframe.h>
+#include <qurl.h>
+
 
 ArkViewer::ArkViewer( QWidget * parent, const char * name )
 	: KDialogBase( parent, name, false, QString::null, Close ), m_part( 0 )
 {
 	m_widget = new QVBox( this );
+	m_widget->layout()->setSpacing( 10 );
 
 	connect( this, SIGNAL( finished() ), this, SLOT( slotFinished() ) );
 
@@ -54,11 +61,9 @@ void ArkViewer::slotFinished()
 
 bool ArkViewer::view( const QString& filename )
 {
-	kdDebug( 1601 ) << "ArkViewer::view(): viewing " << filename << endl;
-
 	KURL u( filename );
 
-	QString mimetype = KMimeType::findByURL( u, 0, true )->name();
+	KMimeType::Ptr mimetype = KMimeType::findByURL( u, 0, true );
 
 	setCaption( u.fileName() );
 
@@ -67,9 +72,23 @@ bool ArkViewer::view( const QString& filename )
 		size = QSize(560, 400);
 	setInitialSize( size );
 
-	kdDebug( 1601 ) << "ArkViewer::view(): mimetype = " << mimetype << endl;
+	QFrame *header = new QFrame( m_widget );
+	QHBoxLayout *headerLayout = new QHBoxLayout( header );
+	headerLayout->setAutoAdd( true );
 
-	m_part = KParts::ComponentFactory::createPartInstanceFromQuery<KParts::ReadOnlyPart>( mimetype, QString::null, m_widget, 0, this );
+	QLabel *iconLabel = new QLabel( header );
+	iconLabel->setPixmap( mimetype->pixmap( KIcon::Desktop ) );
+	iconLabel->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum );
+
+	QVBox *headerRight = new QVBox( header );
+	new QLabel( QString( "<qt><b>%1</b></qt>" )
+	                     .arg( QUrl( filename ).fileName() ), headerRight
+	          );
+	new QLabel( mimetype->comment(), headerRight );
+
+	header->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Maximum );
+
+	m_part = KParts::ComponentFactory::createPartInstanceFromQuery<KParts::ReadOnlyPart>( mimetype->name(), QString::null, m_widget, 0, this );
 
 	if ( m_part )
 	{
