@@ -1,4 +1,3 @@
-//  -*- mode: c++; c-basic-offset: 4; -*-
 /*
 
   ark -- archiver for the KDE project
@@ -24,7 +23,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -35,7 +34,6 @@
 #include <ktempdir.h>
 
 #include <qvbox.h>
-
 #include "arch.h"
 
 class QPoint;
@@ -45,9 +43,9 @@ class QLabel;
 class QListViewItem;
 class QDragMoveEvent;
 class QDropEvent;
+
 class KPopupMenu;
 class KProcess;
-class KConfig;
 class KURL;
 class KRun;
 class KTempFile;
@@ -55,7 +53,6 @@ class KTempDir;
 class KToolBar;
 
 class FileListView;
-class FileLVI;
 class SearchBar;
 
 
@@ -80,29 +77,12 @@ public:
 
     QString tmpDir() const { return m_tmpDir ? m_tmpDir->name() : QString::null; }
 
-    FileListView * fileList() const { return archiveContent; }
+    FileListView * fileList() const { return m_fileListView; }
+    SearchBar    * searchBar() const { return m_searchBar; }
     Arch * archive() const { return arch; }
     ArchType archiveType() const { return m_archType; }
     int numSelectedFiles() const { return m_nNumSelectedFiles; }
 
-    /**
-     * Sets up the column headers for the file list. Clears previous entries before adding new ones.
-     * @param _headers A list of headers to add.
-     * @param _rightAlignCols An array of ints representing the columns ro right-align
-     * @param _numColsToAlignRight Size of _rightAlignCols
-     */
-    void setHeaders( QStringList* _headers, int* _rightAlignCols, int _numColsToAlignRight );
-
-    /**
-     * Clears all headers from the file list.
-     */
-    void clearHeaders();
-
-    /**
-     * Brings up a dialog showing the results returned by the last cmdline tool.
-     */
-    void viewShellOutput();
-
     /**
      * Miscellaneous tasks involved in closing an archive.
      */
@@ -119,7 +99,7 @@ public:
     virtual void setArchivePopupEnabled( bool b );
 
     virtual void extractTo( const KURL & targetDirectory, const KURL & archive, bool bGuessName );
-    virtual void addToArchive( const KURL::List & filesToAdd, const KURL & archive = KURL() );
+    virtual bool addToArchive( const KURL::List & filesToAdd, const KURL & archive = KURL() );
     void convertTo( const KURL & u );
 
     bool isModified() { return m_modified; }
@@ -127,18 +107,12 @@ public:
 
 public slots:
     void file_open( const KURL& url);
-    void edit_view_last_shell_output();
     virtual void file_close();
     virtual void file_new();
     void slotShowSearchBarToggled( bool b );
     void showSettings();
 
 protected slots:
-    void edit_select();
-    void edit_selectAll();
-    void edit_deselectAll();
-    void edit_invertSel();
-
     void action_add();
     void action_add_dir();
     void action_view();
@@ -148,7 +122,7 @@ protected slots:
     void action_edit();
 
     void doPopup(QListViewItem *, const QPoint &, int); // right-click menus
-    void viewFile(); // doubleClick view files
+    void viewFile(QListViewItem*); // doubleClick view files
 
     void slotSelectionChanged();
     void slotOpen(Arch *, bool, const QString &, int);
@@ -158,7 +132,6 @@ protected slots:
     void slotExtractRemoteDone(KIO::Job *job);
     void slotAddDone(bool);
     void slotEditFinished(KProcess *);
-    void selectByPattern(const QString & _pattern);
 signals:
     void openURLRequest( const KURL & url );
     void request_file_quit();
@@ -204,7 +177,7 @@ private: // methods
     // ask user whether to create a real archive from a compressed file
     // returns filename if so. Otherwise, empty.
     KURL askToCreateRealArchive();
-    Arch * getNewArchive( const QString & _fileName );
+    Arch * getNewArchive( const QString & _fileName, const QString& _mimetype = QString() );
     void createRealArchive( const QString &strFilename,
                             const QStringList & filesToAdd = QStringList() );
     KURL getCreateFilename( const QString & _caption,
@@ -260,7 +233,7 @@ protected:
     void newCaption(const QString& filename);
     void createFileListView();
 
-    void createArchive(const QString & name);
+    bool createArchive(const QString & name);
     void openArchive(const QString & name);
 
     void showCurrentFile();
@@ -315,12 +288,12 @@ private: // data
     KToolBar  * m_searchToolBar;
     SearchBar * m_searchBar;
 
-    Arch *arch;
-    QString m_strArchName;
-    KURL m_realURL;
-    FileListView *archiveContent;
-    KURL m_url;
+    Arch   * arch;
+    QString  m_strArchName;
+    KURL     m_realURL;
+    KURL     m_url;
     ArchType m_archType;
+    FileListView * m_fileListView;
 
     KIO::filesize_t m_nSizeOfFiles;
     KIO::filesize_t m_nSizeOfSelectedFiles;
