@@ -1,5 +1,4 @@
 /*
-  -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*-
 
 ark: A program for modifying archives via a GUI.
 
@@ -23,7 +22,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -66,44 +65,61 @@ static KCmdLineOptions option[] =
 extern "C" KDE_EXPORT int kdemain( int argc, char *argv[]  )
 {
 	KAboutData aboutData( "ark", I18N_NOOP( "Ark" ),
-			"2.5.2", I18N_NOOP("KDE Archiving tool"), 
-			KAboutData::License_GPL,
-			I18N_NOOP( "(c) 1997-2005, The Various Ark Developers" ) );
-	
-	aboutData.addAuthor( "Henrique Pinto", I18N_NOOP( "Maintainer" ), "henrique.pinto@kdemail.net" );
-	aboutData.addAuthor( "Helio Chissini de Castro", I18N_NOOP( "Former maintainer" ), "helio@conectiva.com.br" );
-	aboutData.addAuthor( "Georg Robbers", 0, "Georg.Robbers@urz.uni-hd.de" );
-	aboutData.addAuthor( "Roberto Selbach Teixeira", 0, "maragato@kde.org" );
-	aboutData.addAuthor( "Francois-Xavier Duranceau", 0, "duranceau@kde.org" );
-	aboutData.addAuthor( "Emily Ezust (Corel Corporation)", 0, "emilye@corel.com" );
-	aboutData.addAuthor( "Michael Jarrett (Corel Corporation)", 0, "michaelj@corel.com" );
-	aboutData.addAuthor( "Robert Palmbos", 0, "palm9744@kettering.edu" );
+	                      "2.6.1", I18N_NOOP( "KDE Archiving tool" ),
+	                      KAboutData::License_GPL,
+	                      I18N_NOOP( "(c) 1997-2006, The Various Ark Developers" )
+	                    );
+
+	aboutData.addAuthor( "Henrique Pinto",
+	                     I18N_NOOP( "Maintainer" ),
+	                     "henrique.pinto@kdemail.net" );
+	aboutData.addAuthor( "Helio Chissini de Castro",
+	                     I18N_NOOP( "Former maintainer" ),
+	                     "helio@kde.org" );
+	aboutData.addAuthor( "Georg Robbers",
+	                     0,
+	                     "Georg.Robbers@urz.uni-hd.de" );
+	aboutData.addAuthor( "Roberto Selbach Teixeira",
+	                     0,
+	                     "maragato@kde.org" );
+	aboutData.addAuthor( "Francois-Xavier Duranceau",
+	                     0,
+	                     "duranceau@kde.org" );
+	aboutData.addAuthor( "Emily Ezust (Corel Corporation)",
+	                     0,
+	                     "emilye@corel.com" );
+	aboutData.addAuthor( "Michael Jarrett (Corel Corporation)",
+	                     0,
+	                     "michaelj@corel.com" );
+	aboutData.addAuthor( "Robert Palmbos",
+	                     0,
+	                     "palm9744@kettering.edu" );
+
+	aboutData.addCredit( "Bryce Corkins",
+	                     I18N_NOOP( "Icons" ),
+	                     "dbryce@attglobal.net" );
+	aboutData.addCredit( "Liam Smit",
+	                     I18N_NOOP( "Ideas, help with the icons" ),
+	                     "smitty@absamail.co.za" );
 
-	aboutData.addCredit( "Bryce Corkins", I18N_NOOP( "Icons" ), "dbryce@attglobal.net" );
-	aboutData.addCredit( "Liam Smit", I18N_NOOP( "Ideas, help with the icons" ), "smitty@absamail.co.za" );
-	
 	KCmdLineArgs::init( argc, argv, &aboutData );
 	KCmdLineArgs::addCmdLineOptions( option );
 	KCmdLineArgs::addTempFileOption();
-	
-	kdDebug( 1601 ) << "Starting ark. argc=" << argc << "  First arg: " << (argc == 2 ? argv[1] : 0) << endl;
-    if( argc > 2 )
-        for( int c = 1; c<argc; c++ )
-            kdDebug( 1601 ) << "arg[" << c << "] " << argv[ c ] << endl;
-	
+
 	if ( !ArkApplication::start() )
 	{
 		// Already running!
 		kdDebug( 1601 ) << "Already running" << endl;
 		exit( 0 );
 	}
-	
+
 	if ( ArkApplication::getInstance()->isRestored() )
 	{
 		kdDebug( 1601 ) << "In main: Restore..." << endl;
-		RESTORE( ArkTopLevelWindow );
+		RESTORE( MainWindow );
 	}
-	kdDebug( 1601 ) << "Starting ark..." << endl;
-	
+
 	return ArkApplication::getInstance()->exec();
 }
+
+// kate: space-indent off; tab-width 4;
