@@ -19,7 +19,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -56,7 +56,7 @@ RarArch::RarArch( ArkWidget *_gui, const QString & _fileName )
   // Check if rar is available
   bool have_rar = !KGlobal::dirs()->findExe( "rar" ).isNull();
 
-  if ( have_rar ) 
+  if ( have_rar )
   {
     // If it is, then use it as archiver and unarchiver
     m_archiver_program = m_unarchiver_program = "rar";
@@ -67,7 +67,7 @@ RarArch::RarArch( ArkWidget *_gui, const QString & _fileName )
     m_archiver_program = m_unarchiver_program = "unrar";
     setReadOnly( true );
   }
-  
+
   verifyUtilityIsAvailable( m_archiver_program );
 
   m_headerString = "-------------------------------------------------------------------------------";
@@ -115,9 +115,9 @@ void RarArch::open()
   m_header_removed = false;
   m_finished = false;
 
-  KProcess *kp = new KProcess;
+  KProcess *kp = m_currentProcess = new KProcess;
   *kp << m_archiver_program << "v" << "-c-" << m_filename;
-  
+
   connect( kp, SIGNAL( receivedStdout(KProcess*, char*, int) ),
            SLOT( slotReceivedTOC(KProcess*, char*, int) ) );
   connect( kp, SIGNAL( receivedStderr(KProcess*, char*, int) ),
@@ -134,25 +134,18 @@ void RarArch::open()
 
 void RarArch::setHeaders()
 {
-  QStringList list;
-  list.append( FILENAME_STRING );
-  list.append( SIZE_STRING );
-  list.append( PACKED_STRING );
-  list.append( RATIO_STRING );
-  list.append( TIMESTAMP_STRING );
-  list.append( PERMISSION_STRING );
-  list.append( CRC_STRING );
-  list.append( METHOD_STRING );
-  list.append( VERSION_STRING );
+  ColumnList list;
+  list.append( FILENAME_COLUMN );
+  list.append( SIZE_COLUMN );
+  list.append( PACKED_COLUMN );
+  list.append( RATIO_COLUMN );
+  list.append( TIMESTAMP_COLUMN );
+  list.append( PERMISSION_COLUMN );
+  list.append( CRC_COLUMN );
+  list.append( METHOD_COLUMN );
+  list.append( VERSION_COLUMN );
 
-  // which columns to align right
-  int *alignRightCols = new int[3];
-  alignRightCols[0] = 1;
-  alignRightCols[1] = 2;
-  alignRightCols[2] = 3;
-
-  m_gui->setHeaders( &list, alignRightCols, 3 );
-  delete [] alignRightCols;
+  emit headers( list );
 }
 
 void RarArch::create()
@@ -173,33 +166,33 @@ void RarArch::addDir( const QString & _dirName )
 
 void RarArch::addFile( const QStringList & urls )
 {
-  KProcess *kp = new KProcess;
-  
+  KProcess *kp = m_currentProcess = new KProcess;
+
   kp->clearArguments();
   *kp << m_archiver_program;
 
-  if ( Settings::replaceOnlyWithNewer() )
+  if ( ArkSettings::replaceOnlyWithNewer() )
     *kp << "u";
   else
     *kp << "a";
 
-  if ( Settings::rarStoreSymlinks() )
+  if ( ArkSettings::rarStoreSymlinks() )
     *kp << "-ol";
-  if ( Settings::rarRecurseSubdirs() )
+  if ( ArkSettings::rarRecurseSubdirs() )
     *kp << "-r";
 
   *kp << m_filename;
 
   KURL dir( urls.first() );
   QDir::setCurrent( dir.directory() );
-  
+
   QStringList::ConstIterator iter;
   for ( iter = urls.begin(); iter != urls.end(); ++iter )
   {
     KURL url( *iter );
     *kp << url.fileName();
   }
-  
+
   connect( kp, SIGNAL( receivedStdout(KProcess*, char*, int) ),
            SLOT( slotReceivedOutput(KProcess*, char*, int) ) );
   connect( kp, SIGNAL( receivedStderr(KProcess*, char*, int) ),
@@ -214,22 +207,24 @@ void RarArch::addFile( const QStringList & urls )
   }
 }
 
-void RarArch::unarchFile( QStringList *fileList, const QString & destDir,
-                          bool /*viewFriendly*/ )
+void RarArch::unarchFileInternal()
 {
-  if ( destDir.isEmpty() || destDir.isNull() )
+  if ( m_destDir.isEmpty() || m_destDir.isNull() )
   {
     kdError( 1601 ) << "There was no extract directory given." << endl;
     return;
   }
 
-  KProcess *kp = new KProcess;
+  KProcess *kp = m_currentProcess = new KProcess;
   kp->clearArguments();
 
   // extract (and maybe overwrite)
   *kp << m_unarchiver_program << "x";
 
-  if ( !Settings::extractOverwrite() )
+  if ( !m_password.isEmpty() )
+    *kp << "-p=" + m_password;
+
+  if ( !ArkSettings::extractOverwrite() )
   {
     *kp << "-o+";
   }
@@ -242,16 +237,16 @@ void RarArch::unarchFile( QStringList *fileList, const QString & destDir,
 
   // if the file list is empty, no filenames go on the command line,
   // and we then extract everything in the archive.
-  if ( fileList )
+  if ( m_fileList )
   {
     QStringList::Iterator it;
-    for ( it = fileList->begin(); it != fileList->end(); ++it )
+    for ( it = m_fileList->begin(); it != m_fileList->end(); ++it )
     {
       *kp << (*it);
     }
   }
 
-  *kp << destDir ;
+  *kp << m_destDir ;
 
   connect( kp, SIGNAL( receivedStdout(KProcess*, char*, int) ),
            SLOT( slotReceivedOutput(KProcess*, char*, int) ) );
@@ -267,16 +262,21 @@ void RarArch::unarchFile( QStringList *fileList, const QString & destDir,
   }
 }
 
+bool RarArch::passwordRequired()
+{
+    return m_lastShellOutput.findRev("password incorrect ?)")+1;
+}
+
 void RarArch::remove( QStringList *list )
 {
   if ( !list )
     return;
 
-  KProcess *kp = new KProcess;
+  KProcess *kp = m_currentProcess = new KProcess;
   kp->clearArguments();
 
   *kp << m_archiver_program << "d" << m_filename;
-  
+
   QStringList::Iterator it;
   for ( it = list->begin(); it != list->end(); ++it )
   {
