@@ -21,7 +21,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -68,28 +68,16 @@ ZipArch::ZipArch( ArkWidget *_gui, const QString & _fileName )
 
 void ZipArch::setHeaders()
 {
-  QStringList list;
-
-  list.append( FILENAME_STRING );
-  list.append( SIZE_STRING );
-  list.append( METHOD_STRING );
-  list.append( PACKED_STRING );
-  list.append( RATIO_STRING );
-  list.append( TIMESTAMP_STRING );
-  list.append( CRC_STRING );
-
-  // which columns to align right
-  int *alignRightCols = new int[6];
-  alignRightCols[0] = 1;
-  alignRightCols[1] = 2;
-  alignRightCols[2] = 3;
-  alignRightCols[3] = 4;
-  alignRightCols[4] = 5;
-  alignRightCols[5] = 6;
-
-  m_gui->setHeaders( &list, alignRightCols, 6 );
-  delete [] alignRightCols;
+  ColumnList list;
+  list.append( FILENAME_COLUMN );
+  list.append( SIZE_COLUMN );
+  list.append( METHOD_COLUMN );
+  list.append( PACKED_COLUMN );
+  list.append( RATIO_COLUMN );
+  list.append( TIMESTAMP_COLUMN );
+  list.append( CRC_COLUMN );
 
+  emit headers( list );
 }
 
 void ZipArch::open()
@@ -100,7 +88,7 @@ void ZipArch::open()
   m_header_removed = false;
   m_finished = false;
 
-  KProcess *kp = new KProcess;
+  KProcess *kp = m_currentProcess = new KProcess;
 
   *kp << m_unarchiver_program << "-v" << m_filename;
 
@@ -129,37 +117,42 @@ void ZipArch::addDir( const QString & _dirName )
 {
   if ( !_dirName.isEmpty() )
   {
-    bool bOldRecVal = Settings::rarRecurseSubdirs();
+    bool bOldRecVal = ArkSettings::rarRecurseSubdirs();
     // must be true for add directory - otherwise why would user try?
-    Settings::setRarRecurseSubdirs( true );
+    ArkSettings::setRarRecurseSubdirs( true );
 
     QStringList list;
     list.append( _dirName );
     addFile( list );
-    Settings::setRarRecurseSubdirs( bOldRecVal ); // reset to old val
+    ArkSettings::setRarRecurseSubdirs( bOldRecVal ); // reset to old val
   }
 }
 
 void ZipArch::addFile( const QStringList &urls )
 {
-  KProcess *kp = new KProcess;
+  KProcess *kp = m_currentProcess = new KProcess;
   kp->clearArguments();
 
   *kp << m_archiver_program;
 
-  if ( Settings::rarRecurseSubdirs() )
+  if ( !m_password.isEmpty() )
+    *kp << "-P" << m_password;
+
+  if ( ArkSettings::rarRecurseSubdirs() )
     *kp << "-r";
 
-  if ( Settings::rarStoreSymlinks() )
+  if ( ArkSettings::rarStoreSymlinks() )
     *kp << "-y";
 
-  if ( Settings::forceMSDOS() )
+  if ( ArkSettings::forceMSDOS() )
     *kp << "-k";
-  if ( Settings::convertLF2CRLF() )
+  if ( ArkSettings::convertLF2CRLF() )
     *kp << "-l";
 
-  if ( Settings::replaceOnlyWithNewer() )
+  if ( ArkSettings::replaceOnlyWithNewer() )
     *kp << "-u";
+  else
+
 
   *kp << m_filename;
 
@@ -186,47 +179,50 @@ void ZipArch::addFile( const QStringList &urls )
   }
 }
 
-void ZipArch::unarchFile( QStringList *fileList, const QString & destDir,
-                         bool viewFriendly )
+void ZipArch::unarchFileInternal()
 {
   // if fileList is empty, all files are extracted.
   // if destDir is empty, abort with error.
-
-  if ( destDir.isEmpty() || destDir.isNull() )
+  if ( m_destDir.isEmpty() || m_destDir.isNull() )
   {
     kdError( 1601 ) << "There was no extract directory given." << endl;
     return;
   }
 
-  KProcess *kp = new KProcess;
+  KProcess *kp = m_currentProcess = new KProcess;
   kp->clearArguments();
 
   *kp << m_unarchiver_program;
 
-  if ( Settings::extractJunkPaths() && !viewFriendly )
+  if ( !m_password.isEmpty() )
+    *kp << "-P" << m_password;
+
+  if ( ArkSettings::extractJunkPaths() && !m_viewFriendly )
     *kp << "-j" ;
 
-  if ( Settings::rarToLower() )
+  if ( ArkSettings::rarToLower() )
     *kp << "-L";
 
-  if ( Settings::extractOverwrite() )
+  if ( ArkSettings::extractOverwrite() )
     *kp << "-o";
+  else
+    *kp << "-n";
 
   *kp << m_filename;
 
   // if the list is empty, no filenames go on the command line,
   // and we then extract everything in the archive.
-  if ( fileList )
+  if ( m_fileList )
   {
     QStringList::Iterator it;
-    
-    for ( it = fileList->begin(); it != fileList->end(); ++it )
+
+    for ( it = m_fileList->begin(); it != m_fileList->end(); ++it )
     {
       *kp << (*it);
     }
   }
 
-  *kp << "-d" << destDir;
+  *kp << "-d" << m_destDir;
 
   connect( kp, SIGNAL( receivedStdout(KProcess*, char*, int) ),
            SLOT( slotReceivedOutput(KProcess*, char*, int) ) );
@@ -242,16 +238,21 @@ void ZipArch::unarchFile( QStringList *fileList, const QString & destDir,
   }
 }
 
+bool ZipArch::passwordRequired()
+{
+    return m_lastShellOutput.findRev("unable to get password\n")!=-1 || m_lastShellOutput.endsWith("password inflating\n") || m_lastShellOutput.findRev("password incorrect--reenter:")!=-1 || m_lastShellOutput.endsWith("incorrect password\n");
+}
+
 void ZipArch::remove( QStringList *list )
 {
   if ( !list )
     return;
 
-  KProcess *kp = new KProcess;
+  KProcess *kp = m_currentProcess = new KProcess;
   kp->clearArguments();
 
   *kp << m_archiver_program << "-d" << m_filename;
-  
+
   QStringList::Iterator it;
   for ( it = list->begin(); it != list->end(); ++it )
   {
