@@ -1,5 +1,5 @@
 /*
-    $Id: kcalcdisplay.cpp 381208 2005-01-22 13:42:57Z wheeler $
+    $Id: kcalcdisplay.cpp 497672 2006-01-13 14:17:20Z kniederk $
 
     KCalc, a scientific calculator for the X window system using the
     Qt widget libraries, available at no cost at http://www.troll.no
@@ -19,19 +19,22 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
+
 #include <errno.h>
 #include <stdlib.h>
 #include <ctype.h>
 
 #include <qclipboard.h>
+#include <qpainter.h>
 #include <qregexp.h>
 
 #include <kglobal.h>
 #include <klocale.h>
 #include <knotifyclient.h>
+#include "kcalc_settings.h"
 #include "kcalcdisplay.h"
 #include "kcalcdisplay.moc"
 
@@ -40,7 +43,7 @@ static CALCAMNT toDouble(const QString &s, bool &ok)
 {
 	char *ptr = 0;
 	errno = 0;
-	CALCAMNT result = (CALCAMNT) STRTOD(s.latin1(),&ptr);
+	CALCAMNT result = static_cast<CALCAMNT>(STRTOD(s.latin1(),&ptr));
 
 	// find first non-space character for check below
 	while (ptr != 0 && *ptr != '\0' && isspace(*ptr)) {
@@ -54,9 +57,10 @@ static CALCAMNT toDouble(const QString &s, bool &ok)
 
 KCalcDisplay::KCalcDisplay(QWidget *parent, const char *name)
   :QLabel(parent,name), _beep(false), _groupdigits(false), _button(0), _lit(false),
-   _num_base(NB_DECIMAL), _input_limit(0),
+   _num_base(NB_DECIMAL),
    _display_size(DEC_SIZE), _precision(9),
-   _fixed_precision(-1), _error(false), selection_timer(new QTimer)
+   _fixed_precision(-1), _display_amount(0),
+   selection_timer(new QTimer)
 {
 	setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
 	setAlignment(AlignRight | AlignVCenter);
@@ -82,10 +86,9 @@ bool KCalcDisplay::sendEvent(Event const event)
 	switch(event)
 	{
 	case EventReset:
-		_error = false;
-		_display_amount = 0.0;
+		_display_amount = 0;
 		_str_int = "0";
-		_str_int_exp = (char *)0;
+		_str_int_exp = QString::null;
 
 		_eestate = false;
 		_period = false;
@@ -95,12 +98,10 @@ bool KCalcDisplay::sendEvent(Event const event)
 
 		return true;
 	case EventClear:
-		if (! _error) return sendEvent(EventReset);
-		return false;
+		return sendEvent(EventReset);
 	case EventChangeSign:
 		return changeSign();
 	case EventError:
-		_error = true;
 		updateDisplay();
 
 		return true;
@@ -112,25 +113,17 @@ bool KCalcDisplay::sendEvent(Event const event)
 
 void KCalcDisplay::slotCut(void)
 {
-	if(_error  &&  _beep)
-	{
-		KNotifyClient::beep();
-		return;
-	}
-
 	slotCopy();
 	sendEvent(EventReset);
 }
 
 void KCalcDisplay::slotCopy(void)
 {
-	if(_error  &&  _beep)
-	{
-		KNotifyClient::beep();
-		return;
-	}
-
-	QString txt = text();
+	QString txt;
+	if (_num_base != NB_DECIMAL)
+		txt = QLabel::text();
+	else
+		txt = _display_amount.toQString();
 	if (_num_base == NB_HEX)
 		txt.prepend( "0x" );
 	(QApplication::clipboard())->setText(txt, QClipboard::Clipboard);
@@ -141,7 +134,7 @@ void KCalcDisplay::slotPaste(bool bClipboard)
 {
 	QString tmp_str = (QApplication::clipboard())->text(bClipboard ? QClipboard::Clipboard : QClipboard::Selection);
 
-	if (_error  ||  tmp_str.isNull())
+	if (tmp_str.isNull())
 	{
 		if (_beep)  KNotifyClient::beep();
 		return;
@@ -164,7 +157,7 @@ void KCalcDisplay::slotPaste(bool bClipboard)
 			return ;
 		}
 	  
-		setAmount(tmp_result);
+		setAmount(static_cast<double>(tmp_result));
 	} 
 	else
 	{
@@ -239,7 +232,7 @@ void KCalcDisplay::setGroupDigits(bool flag)
 	_groupdigits = flag;
 }
 
-CALCAMNT KCalcDisplay::getAmount(void) const
+KNumber const & KCalcDisplay::getAmount(void) const
 {
 	return _display_amount;
 }
@@ -253,61 +246,31 @@ bool KCalcDisplay::setAmount(const QString &string)
 	
 	if (!was_ok)
 	{
-		tmp_result = (CALCAMNT) (0);
+		tmp_result = static_cast<CALCAMNT>(0);
 		if(_beep) KNotifyClient::beep();
 		return false;
 	}
 	
-	setAmount(tmp_result);
+	setAmount(static_cast<double>(tmp_result));
 
 	return true;
 }
 
-bool KCalcDisplay::setAmount(CALCAMNT new_amount)
+bool KCalcDisplay::setAmount(KNumber const & new_amount)
 {
-	if(_error) return false;
-
 	QString display_str;
 
 	_str_int = "0";
-	_str_int_exp = (char *)0;
+	_str_int_exp = QString::null;
 	_period = false;
 	_neg_sign = false;
 	_eestate = false;
 
-	_display_amount = new_amount;
-
 	if (_num_base != NB_DECIMAL)
 	{
-		CALCAMNT	tmp_round;
-		KCALC_LONG	rounded_num = 0;
+		_display_amount = new_amount.integerPart();
 
-		MODF(_display_amount, &tmp_round);
-
-		if (tmp_round < KCALC_LONG_MIN || tmp_round > KCALC_ULONG_MAX)
-		{
-			sendEvent(EventError);
-			return false;
-		}
-
-		//
-		// We may be in that never-never land where boh numbers
-		// turn from positive to negative - if so then we do
-		// just that, allowing boh negative numbers to be entered
-		// as read (from dumps and the like!)
-		//
-		else if (tmp_round > KCALC_LONG_MAX)
-		{
-			_display_amount = KCALC_LONG_MIN + (tmp_round - KCALC_LONG_MAX - 1);
-			rounded_num = (KCALC_LONG)_display_amount;
-		}
-		else
-		{
-			_display_amount = tmp_round;
-			rounded_num = (KCALC_LONG)_display_amount;
-		}
-
-		display_str = QString::number(rounded_num, _num_base).upper();
+		display_str = QString::number(static_cast<long int>(_display_amount), _num_base).upper();
 		if (display_str.length() > DSP_SIZE)
 		{
 			sendEvent(EventError);
@@ -316,16 +279,15 @@ bool KCalcDisplay::setAmount(CALCAMNT new_amount)
 	}
 	else // _num_base == NB_DECIMAL
 	{
-		// We need to use QCString here since it uses the systems native ::sprintf()
-		// implementation which is more flexible than QString's.
-
-		if (_fixed_precision != -1 && _display_amount <= 1.0e+16)
-			display_str = QCString().sprintf(PRINT_FLOAT, _fixed_precision, _display_amount);
+		_display_amount = new_amount;
+	
+		display_str = _display_amount.toQString(KCalcSettings::precision(), _fixed_precision);
+#if 0
 		else if (_display_amount > 1.0e+16)
 			display_str = QCString().sprintf(PRINT_LONG_BIG, _precision + 1, _display_amount);
 		else
 			display_str = QCString().sprintf(PRINT_LONG_BIG, _precision, _display_amount);
-
+#endif
 		if (display_str.length() > DSP_SIZE)
 		{
 			sendEvent(EventError);
@@ -333,7 +295,6 @@ bool KCalcDisplay::setAmount(CALCAMNT new_amount)
 		}
 	}
 
-	Q_ASSERT(_error == false);
 	setText(display_str);
 	return true;
 	
@@ -343,34 +304,9 @@ void KCalcDisplay::setText(QString const &string)
 {
 	QString localizedString = string;
 
-	// Only treat digits (skip the sign)
-	int first_digit = localizedString.find(QRegExp("\\d"));
-	
 	// If we aren't in decimal mode, we don't need to modify the string
-	if (_num_base == NB_DECIMAL  &&  !_error
-	    &&  _groupdigits  &&  first_digit != -1)
-	{
-		// Obtain decimal symbol and thousands separator
-		QString decimalSymbol = KGlobal::locale()->decimalSymbol();
-		QString thousandsSeparator = KGlobal::locale()->thousandsSeparator();
-		// Replace dot with locale decimal separator
-		localizedString.replace(QChar('.'),decimalSymbol);
-	
-		// Insert the thousand separators
-                int decimalSymbolPos = localizedString.find(decimalSymbol);
-                int eSymbolPos = localizedString.find('e');
-
-                if (-1 == decimalSymbolPos)
-                    decimalSymbolPos = localizedString.length();
-                if (-1 == eSymbolPos)
-                    eSymbolPos = localizedString.length();
-
-                int pos = QMIN((decimalSymbolPos - 1), (eSymbolPos-1));
-                
-		for(int index=1;pos>first_digit;pos--,index++)
-			if (index%3 == 0)
-				localizedString.insert(pos,thousandsSeparator);
-	}
+	if (_num_base == NB_DECIMAL  &&  _groupdigits)
+	  localizedString = KGlobal::locale()->formatNumber(string, false, 0); // Note: rounding happened already above!
 
 	QLabel::setText(localizedString);
 	emit changedText(localizedString);
@@ -378,10 +314,12 @@ void KCalcDisplay::setText(QString const &string)
 
 QString KCalcDisplay::text() const
 {
-	if (_num_base != NB_DECIMAL || _error)
+	if (_num_base != NB_DECIMAL)
 		return QLabel::text();
+	QString display_str = _display_amount.toQString(KCalcSettings::precision());
 
-	return QCString().sprintf(PRINT_LONG_BIG, 40, _display_amount);
+	return display_str;
+	//	return QCString().sprintf(PRINT_LONG_BIG, 40, _display_amount);
 }
 
 /* change representation of display to new base (i.e. binary, decimal,
@@ -390,7 +328,7 @@ QString KCalcDisplay::text() const
    being set with "setAmount"). Return value is the new base. */
 int KCalcDisplay::setBase(NumBase new_base)
 {
-	CALCAMNT tmp_val = getAmount();
+	CALCAMNT tmp_val = static_cast<double>(getAmount());
 
 	switch(new_base)
 	{
@@ -398,36 +336,38 @@ int KCalcDisplay::setBase(NumBase new_base)
 		_num_base	= NB_HEX;
 		_display_size	= HEX_SIZE;
 		_period 	= false;
-		_input_limit		= sizeof(KCALC_LONG)*2;
 		break;
 	case NB_DECIMAL:
 		_num_base	= NB_DECIMAL;
 		_display_size	= DEC_SIZE;
-		_input_limit		= 0;
 		break;
 	case NB_OCTAL:
 		_num_base	= NB_OCTAL;
 		_display_size	= OCT_SIZE;
 		_period 	= false;
-		_input_limit		= 11;
 		break;
 	case NB_BINARY:
 		_num_base	= NB_BINARY;
 		_display_size	= BIN_SIZE;
 		_period 	= false;
-		_input_limit		= 32;
 		break;
 	default: // we shouldn't ever end up here
 		_num_base	= NB_DECIMAL;
 		_display_size	= DEC_SIZE;
-		_input_limit		= 0;
 	}
 
-	setAmount(tmp_val);
+	setAmount(static_cast<double>(tmp_val));
 	
 	return _num_base;
 }
 
+void KCalcDisplay::setStatusText(uint i, const QString& text)
+{
+	if (i < NUM_STATUS_TEXT)
+		_str_status[i] = text;
+	update();
+}
+
 bool KCalcDisplay::updateDisplay(void)
 {
 	// Put sign in front.
@@ -439,12 +379,11 @@ bool KCalcDisplay::updateDisplay(void)
 
 	switch(_num_base)
 	{
-	bool tmp_flag;
 	case NB_BINARY:
 		Q_ASSERT(_period == false  && _eestate == false);
-		Q_ASSERT(tmp_string.length() < DSP_SIZE);
+		Q_ASSERT(tmp_string.length() <= DSP_SIZE);
 		setText(tmp_string);
-		_display_amount = STRTOUL(_str_int.latin1(), 0, 2);
+		_display_amount = static_cast<int>(STRTOUL(_str_int.latin1(), 0, 2));
 		if (_neg_sign)
 			_display_amount = -_display_amount;
 		//str_size = cvb(_str_int, boh_work, DSP_SIZE);
@@ -452,18 +391,18 @@ bool KCalcDisplay::updateDisplay(void)
 	  
 	case NB_OCTAL:
 		Q_ASSERT(_period == false  && _eestate == false);
-		Q_ASSERT(tmp_string.length() < DSP_SIZE);
+		Q_ASSERT(tmp_string.length() <= DSP_SIZE);
 		setText(tmp_string);
-		_display_amount = STRTOUL(_str_int.latin1(), 0, 8);
+		_display_amount = static_cast<int>(STRTOUL(_str_int.latin1(), 0, 8));
 		if (_neg_sign)
 			_display_amount = -_display_amount;
 		break;
 		
 	case NB_HEX:
 		Q_ASSERT(_period == false  && _eestate == false);
-		Q_ASSERT(tmp_string.length() < DSP_SIZE);
+		Q_ASSERT(tmp_string.length() <= DSP_SIZE);
 		setText(tmp_string);
-		_display_amount = STRTOUL(_str_int.latin1(), 0, 16);
+		_display_amount = static_cast<int>(STRTOUL(_str_int.latin1(), 0, 16));
 		if (_neg_sign)
 			_display_amount = -_display_amount;
 		break;
@@ -471,28 +410,25 @@ bool KCalcDisplay::updateDisplay(void)
 	case NB_DECIMAL:
 		if(_eestate == false)
 		{
-			Q_ASSERT(tmp_string.length() < DSP_SIZE);
+			Q_ASSERT(tmp_string.length() <= DSP_SIZE);
 			setText(tmp_string);
-			_display_amount = toDouble(tmp_string, tmp_flag);
-			Q_ASSERT(tmp_flag == true);
+			_display_amount = tmp_string;
 		}
 		else
 		{
 			if(_str_int_exp.isNull())
 			{
 				// add 'e0' to display but not to conversion
-				Q_ASSERT(tmp_string.length()+2 < DSP_SIZE);
-				_display_amount = toDouble(tmp_string, tmp_flag);
-				Q_ASSERT(tmp_flag == true);
+				Q_ASSERT(tmp_string.length()+2 <= DSP_SIZE);
+				_display_amount = tmp_string;
 				setText(tmp_string + "e0");
 			}
 			else
 			{
 				tmp_string +=  'e' + _str_int_exp;
-				Q_ASSERT(tmp_string.length() < DSP_SIZE);
+				Q_ASSERT(tmp_string.length() <= DSP_SIZE);
 				setText(tmp_string);
-				_display_amount = toDouble(tmp_string, tmp_flag);
-				Q_ASSERT(tmp_flag == true);
+				_display_amount = tmp_string;
 			}
 		}
 		break;
@@ -501,20 +437,13 @@ bool KCalcDisplay::updateDisplay(void)
 	  return false;
 	}
 
-	if(_error)
-	{
-		if(_beep) KNotifyClient::beep();
-		setText(i18n("Error"));
-		return false;
-	}
-	else
-		return true;
+	return true;
 }
 
 void KCalcDisplay::newCharacter(char const new_char)
 {
 	// error or DISPLAY is already full
-	if (_error || text().length() >= DSP_SIZE)
+	if (text().length() >= _display_size)
 	{
 		if(_beep) KNotifyClient::beep();
 		return;
@@ -694,7 +623,32 @@ bool KCalcDisplay::changeSign(void)
 	return true;
 }
 
-bool KCalcDisplay::getError(void) const
+void KCalcDisplay::drawContents(QPainter *p)
 {
-	return _error;
+	QLabel::drawContents(p);
+
+	// draw the status texts using half of the normal
+	// font size but not smaller than 7pt
+	QFont f(font());
+	f.setPointSize(QMAX((f.pointSize() / 2), 7));
+	p->setFont(f);
+	QFontMetrics fm(f);
+	uint w = fm.width("_____");
+	uint h = fm.height();
+
+	for (uint i = 0; i < NUM_STATUS_TEXT; i++)
+	{
+		p->drawText(5 + i * w, h, _str_status[i]);
+	}
 }
+
+// Return the QLabel's normal size hint vertically expanded
+// by half the font height to make room for the status texts
+QSize KCalcDisplay::sizeHint() const
+{
+	QFont f(font());
+	f.setPointSize(QMAX((f.pointSize() / 2), 7));
+	QFontMetrics fm(f);
+	return QLabel::sizeHint() + QSize(0, fm.height());
+}
+
