@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
  */
 
 #include "konfigurator.h"
@@ -313,7 +313,7 @@ void KWalletConfig::contextMenuRequested(QListViewItem *item, const QPoint& pos,
 	if (item && item->parent()) {
 		KPopupMenu *m = new KPopupMenu(this);
 		m->insertTitle(item->parent()->text(0));
-		m->insertItem("&Delete", this, SLOT(deleteEntry()), Key_Delete);
+		m->insertItem(i18n("&Delete"), this, SLOT(deleteEntry()), Key_Delete);
 		m->popup(pos);
 	}
 }
